%% GW_USW_Joints_metrics_3.m
% This script is under an MIT License.
% Copyright (c) 2018 Pedro Andre Viegas Ochoa de Carvalho
% The licence can be found in the folder:
%   Ultrasonic_guided_wave_and_single-lap_shear_tests_in_defective_ultrasonically_welded_thermoplastic_composite_joints

close all
clear
clc

disp('Correlation analysis of GW signals in ultrasonically welded joints');

%% Check existence of test results
answer = '0';
if exist('GW_cc_metrics.mat','file') == 2
    quest_str = {'Correlation results already exist';...
        'Do you want to generate them again?'};
    answer = questdlg(quest_str,'Test Results','Yes','No','Cancel','No');
end

if strcmp(answer,'Cancel') == 1
    return
end

if strcmp(answer,'Yes') == 1 || exist('GW_cc_metrics.mat','file') == 0
    %% Control variable and memory allocation
    freq_list = {'204','349','486','619'};
    filenum = [1:5,11:14];
    cmp1 = 0;
    cmp2 = 0;
    mtr1 = zeros(25,4,4);
    mtr2 = zeros(20,4,4);
    
    %% For each frequency
    for fn = 1:4
        
        %% List files for each frequency
        curdir = pwd;
        match_name = [curdir,'\**/*',freq_list{fn},'*.mat'];
        file_list = dir(match_name);
        disp(' ');
        disp([freq_list{fn},' kHz']);
        
        %% For each file of batch 2
        for a = 6:10
            
            %% Load file
            fdir = file_list(a).folder;
            fname = file_list(a).name;
            load(fullfile(fdir,fname));
            %disp(fname);
            
            %% Extract information from file string
            % Specimen name
            p = strfind(fname,'_uw');
            if isempty(p) == 0
                sp_name1 = fname(p+1:p+12);
            end
            
            %% Extract signals
            t0 = Tstart;
            dt = Tinterval;
            sigL = RequestedLength;
            t = (t0:dt:(t0 + ((sigL - 1) * dt)))';
            x_act = A(1:sigL);
            x_sen1 = B(1:sigL);
            % Release memory
            clearvars A B Tstart Tinterval RequestedLength
            
            %% Envelope of signal
            [env1,~] = envelope(x_sen1);
            
            %% Phase signal
            ph_1 = angle(hilbert(x_sen1));
            
            %% Compute frequency-domain signal
            fft1 = abs(fftshift(fft(x_sen1)));
            Fs = 1/dt;
            dF = Fs/sigL;
            fvec1 = -Fs/2:dF:Fs/2-dF;
            ki = find(fvec1 >= 0,1); % Find index of freq = 0
            kf = find(fvec1 >= 1e6,1); % Find index of freq = 1 MHz
            fft1 = fft1(ki:kf);
            fvec1 = fvec1(ki:kf);
            
            for b = 1:9
                
                %% Load file
                c = filenum(b);
                fdir = file_list(c).folder;
                fname = file_list(c).name;
                load(fullfile(fdir,fname));
                
                %% Extract information from file string
                % Specimen name
                p = strfind(fname,'_uw');
                if isempty(p) == 0
                    sp_name2 = fname(p+1:p+12);
                end
                disp(['>>> ',sp_name1,' VS ',sp_name2]);
                
                %% Extract signals
                t0 = Tstart;
                dt = Tinterval;
                sigL = RequestedLength;
                t = (t0:dt:(t0 + ((sigL - 1) * dt)))';
                x_act = A(1:sigL);
                x_sen2 = B(1:sigL);
                % Release memory
                clearvars A B Tstart Tinterval RequestedLength

                %% Envelope of signal
                [env2,~] = envelope(x_sen2);
                
                %% Phase signal
                ph_2 = angle(hilbert(x_sen2));
                
                %% Compute frequency-domain signal
                fft2 = abs(fftshift(fft(x_sen2)));
                Fs = 1/dt;
                dF = Fs/sigL;
                fvec2 = -Fs/2:dF:Fs/2-dF;
                ki = find(fvec2 >= 0,1); % Find index of freq = 0
                kf = find(fvec2 >= 1e6,1); % Find index of freq = 1 MHz
                fft2 = fft2(ki:kf);
                fvec2 = fvec2(ki:kf);
                
                %% Compute metrics
                m1 = 1 - abs(corr2(x_sen1,x_sen2));
                m2 = 1 - abs(corr2(env1,env2));
                m3 = 1 - abs(corr2(fft1,fft2));
                m4 = 1 - abs(corr2(ph_1,ph_2));
                
                %% Store metrics
                % Increment case counters
                if b >= 1 && b <= 5
                    cmp1 = cmp1 + 1;
                    mtr1(cmp1,1,fn) = m1;
                    mtr1(cmp1,2,fn) = m2;
                    mtr1(cmp1,3,fn) = m3;
                    mtr1(cmp1,4,fn) = m4;
                    if cmp1 == 25
                       cmp1 = 0; 
                    end
                elseif b >= 6 && b <= 9
                    cmp2 = cmp2 + 1;
                    mtr2(cmp2,1,fn) = m1;
                    mtr2(cmp2,2,fn) = m2;
                    mtr2(cmp2,3,fn) = m3;
                    mtr2(cmp2,4,fn) = m4;
                    if cmp2 == 20
                       cmp2 = 0; 
                    end
                end
                
            end
            
        end
        
    end
    % Save metrics matrices to file
    save('GW_cc_metrics.mat','mtr1','mtr2');
end

%% Load existing file with test results
if strcmp(answer,'No') == 1
    load('GW_cc_metrics.mat');
end

%% Define plot attributes
mrk_str = {'o','*','s','d'};
title_str = {'Time-domain signal',...
   'Time-domain signal envelope','Frequency-domain signal','Phase'};

for m = 1:4
   
    fh = figure();
    
    for f = 1:4
        
        %% Calculate mean and standard deviations
        avg1 = mean(mtr1(:,m,f));
        std1 = std(mtr1(:,m,f))./sqrt(5);
        avg2 = mean(mtr2(:,m,f));
        std2 = std(mtr2(:,m,f))./sqrt(4);
        
        %% Plot
        errorbar([1 3],[avg1 avg2],[std1 std2],...
            mrk_str{f},'MarkerFaceColor','auto');
        axis([0 4 0 1]);
        xticks([1 3]);
        title(title_str{m});
        xlabel('Batch 2 w.r.t.');
        ylabel('Corr Coeff [-]');
        grid on
        
        %% Define legend
        if f == 1
            hold on
        elseif f == 4
           legend('204 kHz','349 kHz','486 kHz','619 kHz');
        end
        
    end
    
end